# Figure_1_Share_of_DL_loans.r

# Created by Ryan Banerjee
# Date: 8.12.2022
# Purpose: Replicate Figure 1 from Banerjee and Serena "Dampening the financial accelerator? Direct lenders and monetary policy"

rm(list=ls())
setwd("C:/JMCB_Direct_Lenders") #


library(data.table)
library(readxl)
library(readstata13)
library(tidyverse)
library(patchwork)
library(wesanderson)


winsor <- function (x, fraction=.05, tails=NA) {
  if(length(fraction) != 1 || fraction < 0 ||
     fraction > 0.5) {
    stop("bad value for 'fraction'")
  }
  lim <- quantile(x, probs=c(fraction, 1-fraction),na.rm=T)
  if (!(tails%in%c("twosided","lower","upper"))) {
    stop("bad value for 'tails'")
  }
  if (tails=="twosided") {
    x[ x < lim[1] ] <- lim[1]
    x[ x > lim[2] ] <- lim[2]
  } else if (tails=="lower") {
    x[ x < lim[1] ] <- lim[1]
  } else if (tails=="upper") {
    x[ x > lim[2] ] <- lim[2]
  }
  x
}


# Load in the dealscan data
ds <- read_excel("Data/DealScan_USD_loans.xlsx", sheet="Sheet1") ## USD term loans only - note that "outstanding loans" is really loans issued
setDT(ds)


ds_all <- read_excel("Data/DealScan_USD_all_loans.xlsx", sheet="Sheet1") ## all USD loans "outstanding loans" is really loans issued
setDT(ds_all)


# Load in the names of lenders data
li <- read.dta13(file="Data/Lender_Information.dta")
setDT(li)

df <- fread(file="Data/Dataset_BanerjeeSerena_Regressions_FullyUpdated_columns_for_match.csv")
df <- unique(df)


df2 <- df[, list(Algorithm_SDC_Name, ULender_InvBank, ULender_Bank, ULender_ONonBank)]
df2 <- unique(df2)

### USD Term loans
dt <- merge(df2,ds, by.x="Algorithm_SDC_Name", by.y="lender_parent_name", all.y=T)
dt[, name_lower := tolower(Algorithm_SDC_Name)]
dt[is.na(ULender_ONonBank) & name_lower %like% "bank", ULender_ONonBank := 0]
dt[is.na(ULender_ONonBank) & name_lower %like% "banca", ULender_ONonBank := 0]
dt[is.na(ULender_ONonBank) & name_lower %like% "ING", ULender_ONonBank := 0]
dt[is.na(ULender_ONonBank), ULender_ONonBank :=1]
dt[, rank := frank(-outstanding_loans), by=list(ULender_ONonBank, year)]
dt[, outstanding_loans_w := winsor(outstanding_loans, 0.01, tails="upper")]

### top 100 DLs and Banks 
dt_agg <- dt[rank<=100][, list(loansUSD = sum(outstanding_loans_w,na.rm=T)), by=list(ULender_ONonBank, year)]
dt_agg_w <- dcast(dt_agg, year ~ ULender_ONonBank, value.var="loansUSD")
dt_agg_w[, DL_share := `1`/(`0` + `1`)]




### ALL USD loans
dt_all <- merge(df2,ds_all, by.x="Algorithm_SDC_Name", by.y="parent_name", all.y=T)
dt_all[, name_lower := tolower(Algorithm_SDC_Name)]
dt_all[is.na(ULender_ONonBank) & name_lower %like% "bank", ULender_ONonBank := 0]
dt_all[is.na(ULender_ONonBank) & name_lower %like% "banca", ULender_ONonBank := 0]
dt_all[is.na(ULender_ONonBank) & name_lower %like% "ING", ULender_ONonBank := 0]
dt_all[is.na(ULender_ONonBank), ULender_ONonBank :=1]
dt_all[, rank := frank(-originated_loans), by=list(ULender_ONonBank, year)]
dt_all[, originated_loans_w := winsor(originated_loans, 0.01, tails="upper")]





### top 100 DLs and Banks 
dt_all_agg <- dt_all[rank<=100][, list(loansUSD_all = sum(originated_loans_w,na.rm=T)), by=list(ULender_ONonBank, year)]
dt_all_agg_w <- dcast(dt_all_agg, year ~ ULender_ONonBank, value.var="loansUSD_all")
dt_all_agg_w[, DL_share := `1`/(`0` + `1`)]

## term loan share and all loans in one graph
dt_agg_w[, loan_type := "Term loans"]
dt_all_agg_w[, loan_type := "All loans"]
dt_graph <- rbindlist(list(dt_agg_w, dt_all_agg_w))

p <- ggplot(dt_graph[year>2004 & year<2021], aes(year, DL_share * 100, colour=loan_type, linetype=loan_type)) + 
  geom_line(size=0.5) + 
  theme_classic(base_size=8) + 
  scale_colour_manual(values=wes_palette("Moonrise2")) + 
  scale_linetype_manual(values = c("dotdash", "solid")) +
  ylim(c(26,36)) +
  theme(
    axis.title = element_blank(),
    legend.title = element_blank(),
    legend.position = "bottom",
    legend.margin = margin(t=-5)
    )
p

ggsave("Results/DL_term_loan_issuance_share_all_loans_term_loans.pdf", p, height=2.8, width=3)


