# Figure_3_HFT_monetary_news_shocks.r

# Date: 7.9.2020
# Purpose: Create trailing aggregates of shocks from HFT monetary policy announcements 



rm(list=ls())
setwd("C:/JMCB_Direct_Lenders") #


library(data.table)
library(ggplot2)
library(openxlsx)

library(readstata13)
library(stargazer)
library(rms)
library(tidyverse)
library(tidyquant)
library(plotly)

winsor <- function (x, fraction=.05, tails=NA) {
  if(length(fraction) != 1 || fraction < 0 ||
     fraction > 0.5) {
    stop("bad value for 'fraction'")
  }
  lim <- quantile(x, probs=c(fraction, 1-fraction),na.rm=T)
  if (!(tails%in%c("twosided","lower","upper"))) {
    stop("bad value for 'tails'")
  }
  if (tails=="twosided") {
    x[ x < lim[1] ] <- lim[1]
    x[ x > lim[2] ] <- lim[2]
  } else if (tails=="lower") {
    x[ x < lim[1] ] <- lim[1]
  } else if (tails=="upper") {
    x[ x > lim[2] ] <- lim[2]
  }
  x
}




############################################################################################################################################
## Merge in US interest rate shock data from Cieslak and Shrimpf - need to isolate pure MP shocks from news shocks during announcements  ###
## This data has a longer timeseries than the KSX data for the US
############################################################################################################################################

dat.mpshocks_cs <- openxlsx:::read.xlsx("Data\\win_ret_var_cov_US_rev2.xlsx", sheet="Sheet1")
dat.mpshocks_cs$date <- convertToDate(dat.mpshocks_cs$date)

### Extract 3m, 2y and 10y high-frequency shocks for relevant windows
windowMPD <- "_m15_p15"
windowOther <- "_m15_p90"

dat.mpshocks_cs_selected <- dat.mpshocks_cs[,c("date","year","month","day","Event","Type","L1_MPD","L1_uMPD",
                                               names(dat.mpshocks_cs)[grep(windowMPD,names(dat.mpshocks_cs))],
                                               names(dat.mpshocks_cs)[grep(windowOther,names(dat.mpshocks_cs))])]

dat.mpshocks_cs_selected <- data.table(dat.mpshocks_cs_selected)
setkeyv(dat.mpshocks_cs_selected, c("date"))

### Follow Cieslak and Schrimpf - post 15 minute window for monetary policy decision, post 90 minute window for other announcements

### create a vector of covariances, variances and return names without window info
vars_windows <- c(names(dat.mpshocks_cs)[grep(windowMPD,names(dat.mpshocks_cs))],names(dat.mpshocks_cs)[grep(windowOther,names(dat.mpshocks_cs))]) 
vars <- unique(gsub('.{8}$', '', vars_windows))

for (i in vars) {
  # print(i)
  mpdwindowvar <- paste0(i,windowMPD)
  otherwindowvar <- paste0(i,windowOther)
    dat.mpshocks_cs_selected[,paste(i) := ifelse(Type=="MPD", get(mpdwindowvar), get(otherwindowvar))]
}

### Define news vs. pure monetary policy shock
### follow simple approach: cor(equity, yield) >= 0 is a news shock, cor(equity, yield)<0 is an mpshock
vars_yields <- c("b2","b5","b10","b3m")  #[(grep("^r_", vars))] ## yield variables start with "r_"

for (i in vars_yields) {
  cov <- paste0("cov_eq_",i)
  dat.mpshocks_cs_selected[,paste0("shock_type_name_",i) := ifelse(get(cov)<0, "mp_shock","news_shock")]
  dat.mpshocks_cs_selected[,paste0("shock_type_",i) := ifelse(get(cov)<0, 1,0)]
  varname_yield <- paste0("r_",i)
  varname_shock <- paste0("shock_type_",i)
  dat.mpshocks_cs_selected[,paste0("mp_shock_",i) := get(varname_yield) * get(varname_shock) ]
  dat.mpshocks_cs_selected[,paste0("news_shock_",i) := get(varname_yield) *(1- get(varname_shock)) ]
}


#######################################################################################################
## Read in equity indices for Banks and non-financials
## Used to partition HFT shocks into those when bank stock perform poorly relative to non-financials
#######################################################################################################

equity_us <- openxlsx:::read.xlsx("Data\\Equity_indices_banks_nonfinancials.xlsx", sheet="US")
equity_world <- openxlsx:::read.xlsx("Data\\Equity_indices_banks_nonfinancials.xlsx", sheet="World")
equity_dm <- openxlsx:::read.xlsx("Data\\Equity_indices_banks_nonfinancials.xlsx", sheet="Developedmkts")


## US
names(equity_us) <- c("date", "banks_eq", "nfc_eq")
equity_us$date <- convertToDate(equity_us$date)

equity_us <- data.table(equity_us)
setkeyv(equity_us, "date")
# compute daily returns
equity_us[, dRI_banks_us := banks_eq /shift(banks_eq)-1]
equity_us[, dRI_nfc_us := nfc_eq /shift(nfc_eq)-1]
equity_us[, D_bank_low_us := ifelse(dRI_banks_us < dRI_nfc_us, 1, 0)]
equity_us[, banks_eq := NULL]
equity_us[, nfc_eq := NULL]

## World
names(equity_world) <- c("date", "banks_eq", "nfc_eq")
equity_world$date <- convertToDate(equity_world$date)

equity_world <- data.table(equity_world)
setkeyv(equity_world, "date")
# compute daily returns
equity_world[, dRI_banks_world := banks_eq /shift(banks_eq)-1]
equity_world[, dRI_nfc_world := nfc_eq /shift(nfc_eq)-1]
equity_world[, D_bank_low_world := ifelse(dRI_banks_world < dRI_nfc_world, 1, 0)]
equity_world[, banks_eq := NULL]
equity_world[, nfc_eq := NULL]

## DEveloped markets
names(equity_dm) <- c("date", "banks_eq", "nfc_eq")
equity_dm$date <- convertToDate(equity_dm$date)

equity_dm <- data.table(equity_dm)
setkeyv(equity_dm, "date")
# compute daily returns
equity_dm[, dRI_banks_dm := banks_eq /shift(banks_eq)-1]
equity_dm[, dRI_nfc_dm := nfc_eq /shift(nfc_eq)-1]
equity_dm[, D_bank_low_dm := ifelse(dRI_banks_dm < dRI_nfc_dm, 1, 0)]
equity_dm[, banks_eq := NULL]
equity_dm[, nfc_eq := NULL]

### Merge equity indicators together
equity <- merge(equity_us, equity_world, by="date")
equity <- merge(equity, equity_dm, by="date")

#### Merge equtiy indicators to HFT shock
dat.mpshocks_cs_selected_equity <- merge(dat.mpshocks_cs_selected, equity, by="date", all.x=T)



# Get sign of equity market reaction on announcement
dat.mpshocks_cs_selected_equity[, r_eq_sign := r_eq/abs(r_eq)]

# Days when either a(equity markets rise) and b(bank stocks return less than NFCs) - is a low bank beta day, similarly when equity markets fall and banks stock do relatively better
dat.mpshocks_cs_selected_equity[, D_bank_low_beta_dm := ifelse((r_eq_sign>0 & D_bank_low_dm==1) |(r_eq_sign<0 & D_bank_low_dm==0), 1, 0)]
dat.mpshocks_cs_selected_equity[, D_bank_low_beta_us := ifelse((r_eq_sign>0 & D_bank_low_us==1) |(r_eq_sign<0 & D_bank_low_us==0), 1, 0)]
dat.mpshocks_cs_selected_equity[, D_bank_low_beta_world := ifelse((r_eq_sign>0 & D_bank_low_world==1) |(r_eq_sign<0 & D_bank_low_world==0), 1, 0)]

### Second attempt: On days when equity market rises (Dummy =1 if bank stocks rise by more than NFCs). On days when equity market declined (Dummy =1 if bank stock fall by more) 
vars_yields <- c("b2","b5","b10","b3m")
for (i in vars_yields) {
  for (j in c("us", "world", "dm")){
    varinput <- paste0("mp_shock_", i)
    varinput2 <- paste0("D_bank_low_beta_", j)
    varname <- paste0(varinput,"_bank_low_",j)
    dat.mpshocks_cs_selected_equity[,paste(varname) := get(varinput) * get(varinput2)]
    varinput <- paste0("news_shock_", i)
    varinput2 <- paste0("D_bank_low_beta_", j)
    varname <- paste0(varinput,"_bank_low_",j)
    dat.mpshocks_cs_selected_equity[,paste(varname) := get(varinput) * get(varinput2)]
  }
}



### lags of key shocks
## aggregate shocks to month (news and MP separately) - then merge to main loan data adding zeros for missing months
date <- seq(as.Date("1997-01-01"),as.Date("2019-01-01"), by="day")
dat.mpshocks_cs.agg <- data.frame(date)
dat.mpshocks_cs.agg <- data.table(dat.mpshocks_cs.agg)

dat.mpshocks_cs.agg <- merge(dat.mpshocks_cs.agg, dat.mpshocks_cs_selected_equity, by="date", all.x=T)

### now create lags

### Lagged sums over 30 days, 60 days and 90 days (1 quarter)
### Rolling 25 trailing average of rates

vars_yields <- c("b2","b5","b10","b3m")
for (i in vars_yields) {
  for (j in c(30, 60, 90,120)){
    varinput <- paste0("mp_shock_", i)
    varname <- paste0(varinput,"_",j)
    dat.mpshocks_cs.agg[,paste(varname) := rollapply(get(varinput), j, sum, na.rm=T, fill=NA, align="right")]
    varinput <- paste0("news_shock_", i)
    varname <- paste0(varinput,"_",j)
    dat.mpshocks_cs.agg[,paste(varname) := rollapply(get(varinput), j, sum, na.rm=T, fill=NA, align="right")]
  }
}

## sum over bank_low shocks - ie days when bank stocks underperformed nfcs
vars_yields <- c("b2","b5","b10","b3m")
for (i in vars_yields) {
  for (j in c(30, 60, 90,120)){
    for (k in c("us", "world", "dm")){
      varinput <- paste0("mp_shock_", i, "_bank_low_", k)
      varname <- paste0(varinput,"_",j)
      dat.mpshocks_cs.agg[,paste(varname) := rollapply(get(varinput), j, sum, na.rm=T, fill=NA, align="right")]
      varinput <- paste0("news_shock_", i, "_bank_low_", k)
      varname <- paste0(varinput,"_",j)
      dat.mpshocks_cs.agg[,paste(varname) := rollapply(get(varinput), j, sum, na.rm=T, fill=NA, align="right")]
    }
  }
}



### sum up all shocks over a single day (because sometimes there are two a day)
dat.mpshocks_cs_day <- dat.mpshocks_cs.agg[,.(mp_shock_b3m_30 = sum(mp_shock_b3m_30,na.rm=T),
                                              mp_shock_b2_30 = sum(mp_shock_b2_30,na.rm=T),
                                              mp_shock_b5_30 = sum(mp_shock_b5_30,na.rm=T),
                                              mp_shock_b10_30 = sum(mp_shock_b10_30,na.rm=T),
                                              
                                              mp_shock_b3m_60 = sum(mp_shock_b3m_60,na.rm=T),
                                              mp_shock_b2_60 = sum(mp_shock_b2_60,na.rm=T),
                                              mp_shock_b5_60 = sum(mp_shock_b5_60,na.rm=T),
                                              mp_shock_b10_60 = sum(mp_shock_b10_60,na.rm=T),
                                              
                                              mp_shock_b3m_90 = sum(mp_shock_b3m_90,na.rm=T),
                                              mp_shock_b2_90 = sum(mp_shock_b2_90,na.rm=T),
                                              mp_shock_b5_90 = sum(mp_shock_b5_90,na.rm=T),
                                              mp_shock_b10_90 = sum(mp_shock_b10_90,na.rm=T),
                                              
                                              mp_shock_b3m_120 = sum(mp_shock_b3m_120,na.rm=T),
                                              mp_shock_b2_120 = sum(mp_shock_b2_120,na.rm=T),
                                              mp_shock_b5_120 = sum(mp_shock_b5_120,na.rm=T),
                                              mp_shock_b10_120 = sum(mp_shock_b10_120,na.rm=T),
                                              
                                              news_shock_b3m_30 = sum(news_shock_b3m_30,na.rm=T),
                                              news_shock_b2_30 = sum(news_shock_b2_30,na.rm=T),
                                              news_shock_b5_30 = sum(news_shock_b5_30,na.rm=T),
                                              news_shock_b10_30 = sum(news_shock_b10_30,na.rm=T),
                                              
                                              news_shock_b3m_60 = sum(news_shock_b3m_60,na.rm=T),
                                              news_shock_b2_60 = sum(news_shock_b2_60,na.rm=T),
                                              news_shock_b5_60 = sum(news_shock_b5_60,na.rm=T),
                                              news_shock_b10_60 = sum(news_shock_b10_60,na.rm=T),
                                              
                                              news_shock_b3m_90 = sum(news_shock_b3m_90,na.rm=T),
                                              news_shock_b2_90 = sum(news_shock_b2_90,na.rm=T),
                                              news_shock_b5_90 = sum(news_shock_b5_90,na.rm=T),
                                              news_shock_b10_90 = sum(news_shock_b10_90,na.rm=T),

                                              news_shock_b3m_120 = sum(news_shock_b3m_120,na.rm=T),
                                              news_shock_b2_120 = sum(news_shock_b2_120,na.rm=T),
                                              news_shock_b5_120 = sum(news_shock_b5_120,na.rm=T),
                                              news_shock_b10_120 = sum(news_shock_b10_120,na.rm=T),
                                              
                                              mp_shock_b3m_bank_low_dm_30 = sum(mp_shock_b3m_bank_low_dm_30, na.rm=T),
                                              mp_shock_b3m_bank_low_dm_60 = sum(mp_shock_b3m_bank_low_dm_60, na.rm=T),
                                              mp_shock_b3m_bank_low_dm_90 = sum(mp_shock_b3m_bank_low_dm_90, na.rm=T), 
                                              mp_shock_b3m_bank_low_dm_120 = sum(mp_shock_b3m_bank_low_dm_120, na.rm=T),
                                              
                                              mp_shock_b2_bank_low_dm_30 = sum(mp_shock_b2_bank_low_dm_30, na.rm=T),
                                              mp_shock_b2_bank_low_dm_60 = sum(mp_shock_b2_bank_low_dm_60, na.rm=T),
                                              mp_shock_b2_bank_low_dm_90 = sum(mp_shock_b2_bank_low_dm_90, na.rm=T), 
                                              mp_shock_b2_bank_low_dm_120 = sum(mp_shock_b2_bank_low_dm_120, na.rm=T),
                                              
                                              mp_shock_b5_bank_low_dm_30 = sum(mp_shock_b5_bank_low_dm_30, na.rm=T),
                                              mp_shock_b5_bank_low_dm_60 = sum(mp_shock_b5_bank_low_dm_60, na.rm=T),
                                              mp_shock_b5_bank_low_dm_90 = sum(mp_shock_b5_bank_low_dm_90, na.rm=T), 
                                              mp_shock_b5_bank_low_dm_120 = sum(mp_shock_b5_bank_low_dm_120, na.rm=T),
                                              
                                              mp_shock_b10_bank_low_dm_30 = sum(mp_shock_b10_bank_low_dm_30, na.rm=T),
                                              mp_shock_b10_bank_low_dm_60 = sum(mp_shock_b10_bank_low_dm_60, na.rm=T),
                                              mp_shock_b10_bank_low_dm_90 = sum(mp_shock_b10_bank_low_dm_90, na.rm=T), 
                                              mp_shock_b10_bank_low_dm_120 = sum(mp_shock_b10_bank_low_dm_120, na.rm=T),
                                              
                                              news_shock_b3m_bank_low_dm_30 = sum(news_shock_b3m_bank_low_dm_30, na.rm=T),
                                              news_shock_b3m_bank_low_dm_60 = sum(news_shock_b3m_bank_low_dm_60, na.rm=T),
                                              news_shock_b3m_bank_low_dm_90 = sum(news_shock_b3m_bank_low_dm_90, na.rm=T), 
                                              news_shock_b3m_bank_low_dm_120 = sum(news_shock_b3m_bank_low_dm_120, na.rm=T),
                                              
                                              news_shock_b2_bank_low_dm_30 = sum(news_shock_b2_bank_low_dm_30, na.rm=T),
                                              news_shock_b2_bank_low_dm_60 = sum(news_shock_b2_bank_low_dm_60, na.rm=T),
                                              news_shock_b2_bank_low_dm_90 = sum(news_shock_b2_bank_low_dm_90, na.rm=T), 
                                              news_shock_b2_bank_low_dm_120 = sum(news_shock_b2_bank_low_dm_120, na.rm=T),
                                              
                                              news_shock_b5_bank_low_dm_30 = sum(news_shock_b5_bank_low_dm_30, na.rm=T),
                                              news_shock_b5_bank_low_dm_60 = sum(news_shock_b5_bank_low_dm_60, na.rm=T),
                                              news_shock_b5_bank_low_dm_90 = sum(news_shock_b5_bank_low_dm_90, na.rm=T), 
                                              news_shock_b5_bank_low_dm_120 = sum(news_shock_b5_bank_low_dm_120, na.rm=T),
                                              
                                              news_shock_b10_bank_low_dm_30 = sum(news_shock_b10_bank_low_dm_30, na.rm=T),
                                              news_shock_b10_bank_low_dm_60 = sum(news_shock_b10_bank_low_dm_60, na.rm=T),
                                              news_shock_b10_bank_low_dm_90 = sum(news_shock_b10_bank_low_dm_90, na.rm=T), 
                                              news_shock_b10_bank_low_dm_120 = sum(news_shock_b10_bank_low_dm_120, na.rm=T),
                                              
                                              mp_shock_b3m_0 = sum(mp_shock_b3m,na.rm=T),
                                              mp_shock_b2_0 = sum(mp_shock_b2,na.rm=T),
                                              mp_shock_b5_0 = sum(mp_shock_b5,na.rm=T),
                                              mp_shock_b10_0 = sum(mp_shock_b10,na.rm=T),
                                              
                                              news_shock_b3m_0 = sum(news_shock_b3m,na.rm=T),
                                              news_shock_b2_0 = sum(news_shock_b2,na.rm=T),
                                              news_shock_b5_0 = sum(news_shock_b5,na.rm=T),
                                              news_shock_b10_0 = sum(news_shock_b10,na.rm=T)
                                              
                                              ), by="date"]

### Extract year, month and day for merge with loan issuance
dat.mpshocks_cs_day[, year := as.numeric(format(date, "%Y"))]
dat.mpshocks_cs_day[, month := as.numeric(format(date, "%m"))]
dat.mpshocks_cs_day[, day := as.numeric(format(date, "%d"))]



dat.mpshocks_cs.agg[, crisis:= ifelse(date > as.Date("2008-07-01") & date< as.Date("2009-01-01"), "crisis","notcrisis")]


##### Graphs for paper
## compare raw shocks
p90 <- ggplot(dat.mpshocks_cs.agg %>%
         select(date, news_shock_b2_90, mp_shock_b2_90), aes(date, news_shock_b2_90)) + geom_line(linetype="dotdash") + geom_line(aes(y=mp_shock_b2_90), colour="red") + 
  theme_classic() + 
  xlab(NULL) + ylab("Cumulative interest rate shocks over 90 days (bps)")




## Do same graph by in long format
p <- ggplot(dat.mpshocks_cs.agg %>%
         select(date, news_shock_b2_90, mp_shock_b2_90) %>%
         pivot_longer(!date, names_to="shock_type", values_to="shock") %>%
         mutate(shock_type = ifelse(shock_type=="news_shock_b2_90", "Information shock", "Pure monetary policy shock")) %>%
           mutate(shock_type = factor(shock_type, levels =c("Pure monetary policy shock", "Information shock"))), aes(date, shock, colour=shock_type, linetype=shock_type)) +
  geom_line(size=0.5) + 
  theme_classic(base_size=8) + 
  theme(legend.position = "bottom", 
        legend.title=element_blank()
        # scale_linetype_manual(values=c("solid", "dotdash"))
        ) + 
  xlab(NULL) + ylab("Basis points")

p

ggsave("Results/Graph_90_shock.pdf", p, height=2.8, width=3)

ggplot(dat.mpshocks_cs_selected, aes(date, news_shock_b2)) + geom_line(linetype="dotdash") + geom_line(aes(y=mp_shock_b2), colour="red") + 
  theme_classic() + 
  xlab(NULL) + ylab("Interest rate shocks (bps)")


## Do same graph by in long format
p <- ggplot(dat.mpshocks_cs_selected %>%
              select(date, news_shock_b2, mp_shock_b2) %>%
              pivot_longer(!date, names_to="shock_type", values_to="shock") %>%
              mutate(shock_type = ifelse(shock_type=="news_shock_b2", "Information shock", "Pure monetary policy shock")) %>%
              mutate(shock_type = factor(shock_type, levels =c("Pure monetary policy shock", "Information shock"))), aes(date, shock, colour=shock_type, linetype=shock_type)) +
  geom_line(size=0.5) + 
  theme_classic(base_size=8) + 
  theme(legend.position = "bottom", 
        legend.title=element_blank()
        # scale_linetype_manual(values=c("solid", "dotdash"))
  ) + 
  xlab(NULL) + ylab("Basis points")

p

ggsave("Results/Graph_shock.pdf", p, height=2.8, width=3)

